Package: meta
Title: General Package for Meta-Analysis
Version: 8.5-0
Date: 2026-05-25
Depends: R (>= 4.0.0), metabook (>= 0.2-0)
Imports: metafor (>= 3.0-0), grid, ggplot2, lme4, CompQuadForm, xml2,
        methods, readr, dplyr, magrittr, purrr, stringr, rlang, tibble,
        scales, cli
Suggests: netmeta, BiasedUrn, pimeta, estmeansd, robvis, brglm2,
        writexl, rmarkdown, knitr, ggpubr, gridExtra, gemtc, ragg,
        metadat, svglite
Authors@R: person("Guido", "Schwarzer",
                  role = c("cre", "aut"),
                  email = "guido.schwarzer@uniklinik-freiburg.de",
                  comment = c(ORCID = "0000-0001-6214-9087"))
Author: Guido Schwarzer [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-6214-9087>)
Maintainer: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
URL: https://github.com/guido-s/meta/
        https://link.springer.com/book/10.1007/978-3-319-21416-0
Description: User-friendly general package providing standard methods for meta-analysis and supporting Schwarzer, Carpenter, and Rücker <DOI:10.1007/978-3-319-21416-0>, "Meta-Analysis with R" (2015):
 - common effect and random effects meta-analysis;
 - several plots (forest, funnel, Galbraith / radial, L'Abbe, Baujat, bubble);
 - three-level meta-analysis model;
 - generalised linear mixed model;
 - logistic regression with penalised likelihood for rare events;
 - Hartung-Knapp method for random effects model;
 - Kenward-Roger method for random effects model;
 - prediction interval and density of the prediction distribution;
 - expected proportion of comparable studies with clinically important benefit or harm;
 - statistical tests  for funnel plot asymmetry;
 - trim-and-fill method to evaluate bias in meta-analysis;
 - meta-regression;
 - cumulative meta-analysis and leave-one-out meta-analysis;
 - import data from 'RevMan 5';
 - produce forest plot summarising several (subgroup) meta-analyses.
License: GPL (>= 2)
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 22:18:27 UTC; sc
Repository: CRAN
Date/Publication: 2026-05-25 05:10:02 UTC
Built: R 4.6.0; ; 2026-05-25 07:34:34 UTC; unix
