# Generated by roxygen2: do not edit by hand

S3method("[",longarm)
S3method("[",pairwise)
S3method(as.data.frame,meta)
S3method(barplot,rob)
S3method(baujat,meta)
S3method(blup,meta)
S3method(bubble,metareg)
S3method(cidprop,meta)
S3method(estimates,blup.meta)
S3method(estimates,meta)
S3method(forest,meta)
S3method(forest,metabind)
S3method(forest,metacum)
S3method(forest,metainf)
S3method(funnel,meta)
S3method(labbe,default)
S3method(labbe,metabin)
S3method(labels,meta)
S3method(metabias,cdir)
S3method(metabias,default)
S3method(metabias,meta)
S3method(metabias,rm5)
S3method(metacum,default)
S3method(metacum,meta)
S3method(metainf,default)
S3method(metainf,meta)
S3method(metareg,default)
S3method(metareg,meta)
S3method(nnt,default)
S3method(nnt,meta)
S3method(plot,cidprop)
S3method(plot,meta)
S3method(print,blup.meta)
S3method(print,cdir)
S3method(print,cidprop)
S3method(print,estimates.blup.meta)
S3method(print,estimates.meta)
S3method(print,meta)
S3method(print,metabias)
S3method(print,metacum)
S3method(print,metainf)
S3method(print,nnt.default)
S3method(print,nnt.meta)
S3method(print,rd.default)
S3method(print,rd.meta)
S3method(print,rm5)
S3method(print,rob)
S3method(print,summary.cdir)
S3method(print,summary.meta)
S3method(print,summary.rm5)
S3method(print,trimfill.cdir)
S3method(print,trimfill.rm5)
S3method(radial,default)
S3method(radial,meta)
S3method(rd,default)
S3method(rd,meta)
S3method(subset,longarm)
S3method(subset,pairwise)
S3method(summary,cdir)
S3method(summary,meta)
S3method(summary,metacum)
S3method(summary,metainf)
S3method(summary,rm5)
S3method(trimfill,cdir)
S3method(trimfill,default)
S3method(trimfill,meta)
S3method(trimfill,rm5)
S3method(update,meta)
S3method(weights,meta)
export(.forestArgs)
export(JAMAlabels)
export(VE2logVR)
export(asin2ir)
export(asin2p)
export(backtransf)
export(baujat)
export(blup)
export(bubble)
export(ci)
export(cidprop)
export(cilayout)
export(cor2z)
export(drapery)
export(estimates)
export(forest)
export(forest_dims)
export(funnel)
export(gs)
export(inches2units)
export(labbe)
export(logVR2VE)
export(logit2p)
export(longarm)
export(metaadd)
export(metabias)
export(metabin)
export(metabind)
export(metacont)
export(metacor)
export(metacr)
export(metacum)
export(metagen)
export(metainc)
export(metainf)
export(metamean)
export(metamerge)
export(metaprop)
export(metarate)
export(metareg)
export(nnt)
export(or2smd)
export(p2asin)
export(p2logit)
export(pairwise)
export(radial)
export(rd)
export(read.cdir)
export(read.mtv)
export(read.rm5)
export(rob)
export(settings.meta)
export(setvals)
export(smd2or)
export(traffic_light)
export(transf)
export(trimfill)
export(units2inches)
export(z2cor)
import(metabook)
importFrom(CompQuadForm,farebrother)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_warn)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_map)
importFrom(dplyr,if_else)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,select)
importFrom(dplyr,tibble)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,bmp)
importFrom(grDevices,cairo_pdf)
importFrom(grDevices,cairo_ps)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.set)
importFrom(grDevices,dev.size)
importFrom(grDevices,gray)
importFrom(grDevices,gray.colors)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,pictex)
importFrom(grDevices,png)
importFrom(grDevices,postscript)
importFrom(grDevices,svg)
importFrom(grDevices,tiff)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(grid,arrow)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,convertX)
importFrom(grid,gpar)
importFrom(grid,grid.circle)
importFrom(grid,grid.draw)
importFrom(grid,grid.get)
importFrom(grid,grid.gget)
importFrom(grid,grid.grabExpr)
importFrom(grid,grid.layout)
importFrom(grid,grid.lines)
importFrom(grid,grid.newpage)
importFrom(grid,grid.polygon)
importFrom(grid,grid.rect)
importFrom(grid,grid.text)
importFrom(grid,grid.xaxis)
importFrom(grid,grobTree)
importFrom(grid,popViewport)
importFrom(grid,pushViewport)
importFrom(grid,rectGrob)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,unit.c)
importFrom(grid,viewport)
importFrom(lme4,glmer)
importFrom(magrittr,"%<>%")
importFrom(metafor,baujat)
importFrom(metafor,blup)
importFrom(metafor,confint.rma.mv)
importFrom(metafor,confint.rma.uni)
importFrom(metafor,escalc)
importFrom(metafor,forest)
importFrom(metafor,funnel)
importFrom(metafor,funnel.default)
importFrom(metafor,labbe)
importFrom(metafor,predict.rma)
importFrom(metafor,radial)
importFrom(metafor,regtest)
importFrom(metafor,rma.glmm)
importFrom(metafor,rma.mv)
importFrom(metafor,rma.uni)
importFrom(metafor,to.long)
importFrom(metafor,trimfill)
importFrom(metafor,vcalc)
importFrom(methods,formalArgs)
importFrom(purrr,compact)
importFrom(readr,cols)
importFrom(readr,read_csv)
importFrom(rlang,":=")
importFrom(rlang,enexprs)
importFrom(rlang,expr)
importFrom(rlang,is_bare_list)
importFrom(scales,number_format)
importFrom(stats,as.formula)
importFrom(stats,binom.test)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dt)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,poisson.test)
importFrom(stats,pt)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(stats,weights)
importFrom(stringr,str_pad)
importFrom(tibble,column_to_rownames)
importFrom(tools,file_ext)
importFrom(utils,assignInNamespace)
importFrom(utils,combn)
importFrom(utils,count.fields)
importFrom(utils,find)
importFrom(utils,getFromNamespace)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,unzip)
importFrom(xml2,as_xml_document)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_text)
