
         *========================================*
                      |OpenViBE| |AUTHORS|
         *========================================*

         Contact: http://openvibe.inria.fr/contact/
		 
|People|

OpenViBE staff in alphabetical order

 - Guillermo Andrade-Barroso / Inria
 - Laurent Bonnet / Inria, Mensia
 - Alison Cellard / Inria
 - Jerome Chabrol / Inria 
 - Vincent Delannoy / Inria
 - Arthur Desbois / Inria
 - Agnes Dos Santos / Mensia 
 - Morgane Fauvet / Mensia
 - Nathanael Foy / Inria 
 - Charles Garraud / Inria 
 - Thierry Gaugry / Inria 
 - Anatole Lecuyer / Inria
 - Jozef Legeny / Inria, Mensia
 - Jussi T. Lindgren / Inria 
 - Fabien Lotte / Inria
 - Loic Mahe / Inria
 - Thibaut Monseigne / Inria
 - Baptiste Payan / Inria
 - Benoit Perrin / Mensia
 - Alexis Placet / Mensia
 - Thomas Prampart / Inria
 - Yann Renard / Inria, Mensia
 - Bruno Renier / Inria
 - Cedric Riou / Inria 
 - Guillaume Serri�re / Inria
 
Scientific Steering Committee :
 - Laurent Bougrain / Inria
 - Maureen Clerc / Inria
 - Anatole Lecuyer / Inria
 - Fabien Lotte / Inria
 - Theodore Papadopoulo / Inria

Other contributors :

 - Please see 
   
   http://openvibe.inria.fr/contributors/

 
|End.of.file|

 vim:tw=78:ts=8:ft=help:norl:
