# What is EBML {#ebml}

\page WhatIsEBML What is EBML

This page tries to explain what EBML is and why it is usefull.

- see http://ebml.sourceforge.net/ for more details
- see http://ebml.sourceforge.net/specs/ for specifications
- see http://www.matroska.org/technical/specs/ for even more details

- check libebml for a sample implementation of EBML
- check libmatroska for a sample of library using libebml
  and a concrete use of EBML

\todo Write more documentation on what EBML is

− \subpage ParsingEBML  
− \subpage FormatingEBML
