# OpenViBE Stimulations Code list {#stimulations}

\page Stimulations OpenViBE Stimulations Code

| Name                                                  | Hexadecimal Code | Decimal Code |
| :---------------------------------------------------- | ---------------: | -----------: |
| OVTK_GDF_100_Watt                                     |     0x00000584   |       1412   |
| OVTK_GDF_125_Watt                                     |     0x00000585   |       1413   |
| OVTK_GDF_150_Watt                                     |     0x00000586   |       1414   |
| OVTK_GDF_175_Watt                                     |     0x00000587   |       1415   |
| OVTK_GDF_200_Watt                                     |     0x00000588   |       1416   |
| OVTK_GDF_225_Watt                                     |     0x00000589   |       1417   |
| OVTK_GDF_25_Watt                                      |     0x00000581   |       1409   |
| OVTK_GDF_250_Watt                                     |     0x0000058A   |       1418   |
| OVTK_GDF_275_Watt                                     |     0x0000058B   |       1419   |
| OVTK_GDF_300_Watt                                     |     0x0000058C   |       1420   |
| OVTK_GDF_325_Watt                                     |     0x0000058D   |       1421   |
| OVTK_GDF_350_Watt                                     |     0x0000058E   |       1422   |
| OVTK_GDF_50_Watt                                      |     0x00000582   |       1410   |
| OVTK_GDF_75_Watt                                      |     0x00000583   |       1411   |
| OVTK_GDF_AEP                                          |     0x00000122   |        290   |
| OVTK_GDF_Arm_Movement                                 |     0x00000449   |       1097   |
| OVTK_GDF_Arm_Right_Movement                           |     0x0000044A   |       1098   |
| OVTK_GDF_Artifact_50_60_Hz_Interference               |     0x00000107   |        263   |
| OVTK_GDF_Artifact_Breathing                           |     0x00000108   |        264   |
| OVTK_GDF_Artifact_ECG                                 |     0x00000102   |        258   |
| OVTK_GDF_Artifact_EMG                                 |     0x00000103   |        259   |
| OVTK_GDF_Artifact_EOG_Large                           |     0x00000101   |        257   |
| OVTK_GDF_Artifact_EOG_Small                           |     0x0000010A   |        266   |
| OVTK_GDF_Artifact_Failing_Electrode                   |     0x00000105   |        261   |
| OVTK_GDF_Artifact_Movement                            |     0x00000104   |        260   |
| OVTK_GDF_Artifact_Pulse                               |     0x00000109   |        265   |
| OVTK_GDF_Artifact_Sweat                               |     0x00000106   |        262   |
| OVTK_GDF_Beep                                         |     0x00000311   |        785   |
| OVTK_GDF_Biting                                       |     0x00000446   |       1094   |
| OVTK_GDF_CAHE                                         |     0x00000403   |       1027   |
| OVTK_GDF_Calibration                                  |     0x0000010F   |        271   |
| OVTK_GDF_class10                                      |     0x0000030A   |        778   |
| OVTK_GDF_class11                                      |     0x0000030B   |        779   |
| OVTK_GDF_class5                                       |     0x00000305   |        773   |
| OVTK_GDF_class7                                       |     0x00000307   |        775   |
| OVTK_GDF_class8                                       |     0x00000308   |        776   |
| OVTK_GDF_class9                                       |     0x00000309   |        777   |
| OVTK_GDF_Correct                                      |     0x00000381   |        897   |
| OVTK_GDF_Cross_On_Screen                              |     0x00000312   |        786   |
| OVTK_GDF_CSB                                          |     0x00000404   |       1028   |
| OVTK_GDF_Cue_Unknown_Undefined                        |     0x0000030F   |        783   |
| OVTK_GDF_Down                                         |     0x00000306   |        774   |
| OVTK_GDF_ECG_Fiducial_Point_QRS_Complex               |     0x00000501   |       1281   |
| OVTK_GDF_ECG_P_Wave                                   |     0x00000502   |       1282   |
| OVTK_GDF_ECG_QRS_Complex                              |     0x00000503   |       1283   |
| OVTK_GDF_ECG_R_Point                                  |     0x00000504   |       1284   |
| OVTK_GDF_ECG_T_Wave                                   |     0x00000506   |       1286   |
| OVTK_GDF_ECG_U_Wave                                   |     0x00000507   |       1287   |
| OVTK_GDF_EEG_Idling_EEG_Eyes_Closed                   |     0x00000115   |        277   |
| OVTK_GDF_EEG_Idling_EEG_Eyes_Open                     |     0x00000114   |        276   |
| OVTK_GDF_EEG_K_Complexes                              |     0x00000112   |        274   |
| OVTK_GDF_EEG_Saw_Tooth_Waves                          |     0x00000113   |        275   |
| OVTK_GDF_EEG_Seizure                                  |     0x00000117   |        279   |
| OVTK_GDF_EEG_Sleep_Splindles                          |     0x00000111   |        273   |
| OVTK_GDF_EEG_Spike                                    |     0x00000116   |        278   |
| OVTK_GDF_End_Of_Session                               |     0x000003F2   |       1010   |
| OVTK_GDF_End_Of_Trial                                 |     0x00000320   |        800   |
| OVTK_GDF_Eye_Blink                                    |     0x00000439   |       1081   |
| OVTK_GDF_Eyes_Down                                    |     0x00000434   |       1076   |
| OVTK_GDF_Eyes_Left                                    |     0x00000431   |       1073   |
| OVTK_GDF_Eyes_Right                                   |     0x00000432   |       1074   |
| OVTK_GDF_Eyes_Up                                      |     0x00000433   |       1075   |
| OVTK_GDF_Feedback_Continuous                          |     0x0000030D   |        781   |
| OVTK_GDF_Feedback_Discrete                            |     0x0000030E   |        782   |
| OVTK_GDF_Flashing_Light                               |     0x00000313   |        787   |
| OVTK_GDF_Foot                                         |     0x00000303   |        771   |
| OVTK_GDF_Foot_Movement                                |     0x00000447   |       1095   |
| OVTK_GDF_Foot_Right_Movement                          |     0x00000448   |       1096   |
| OVTK_GDF_Head_Movement                                |     0x00000443   |       1091   |
| OVTK_GDF_Horizontal_Eye_Movement                      |     0x00000435   |       1077   |
| OVTK_GDF_Incorrect                                    |     0x00000382   |        898   |
| OVTK_GDF_Left                                         |     0x00000301   |        769   |
| OVTK_GDF_Left_Hand_Movement                           |     0x00000441   |       1089   |
| OVTK_GDF_Lights_Off                                   |     0x00008420   |      33824   |
| OVTK_GDF_Lights_On                                    |     0x00000420   |       1056   |
| OVTK_GDF_Maximum_Inspiration                          |     0x0000040E   |       1038   |
| OVTK_GDF_Non_Equidistant_Sampling_Value               |     0x00007FFF   |      32767   |
| OVTK_GDF_OAHE                                         |     0x00000401   |       1025   |
| OVTK_GDF_Rejection                                    |     0x000003FF   |       1023   |
| OVTK_GDF_REM                                          |     0x00000415   |       1045   |
| OVTK_GDF_RERA                                         |     0x00000402   |       1026   |
| OVTK_GDF_Right                                        |     0x00000302   |        770   |
| OVTK_GDF_Right_Hand_Movement                          |     0x00000442   |       1090   |
| OVTK_GDF_Rotation_Clockwise                           |     0x00000437   |       1079   |
| OVTK_GDF_Rotation_Counterclockwise                    |     0x00000438   |       1080   |
| OVTK_GDF_SEP                                          |     0x00000123   |        291   |
| OVTK_GDF_Sleep_Hypoventilation                        |     0x00000405   |       1029   |
| OVTK_GDF_SSAEP                                        |     0x00000132   |        306   |
| OVTK_GDF_SSSEP                                        |     0x00000133   |        307   |
| OVTK_GDF_SSVEP                                        |     0x00000131   |        305   |
| OVTK_GDF_Stage_1                                      |     0x00000411   |       1041   |
| OVTK_GDF_Stage_2                                      |     0x00000412   |       1042   |
| OVTK_GDF_Stage_3                                      |     0x00000413   |       1043   |
| OVTK_GDF_Stage_4                                      |     0x00000414   |       1044   |
| OVTK_GDF_Start                                        |     0x00000580   |       1408   |
| OVTK_GDF_Start_Of_Inspiration                         |     0x0000040F   |       1039   |
| OVTK_GDF_Start_Of_New_Segment                         |     0x00007FFE   |      32766   |
| OVTK_GDF_Start_Of_Trial                               |     0x00000300   |        768   |
| OVTK_GDF_Swallowing                                   |     0x00000445   |       1093   |
| OVTK_GDF_TMS                                          |     0x0000012F   |        303   |
| OVTK_GDF_Tongue                                       |     0x00000304   |        772   |
| OVTK_GDF_Tongue_Movement                              |     0x00000444   |       1092   |
| OVTK_GDF_Up                                           |     0x0000030C   |        780   |
| OVTK_GDF_VEP                                          |     0x00000121   |        289   |
| OVTK_GDF_Vertical_Eye_Movement                        |     0x00000436   |       1078   |
| OVTK_GDF_Wake                                         |     0x00000410   |       1040   |
| OVTK_StimulationId_AddedSamplesBegin                  |     0x00008311   |      33553   |
| OVTK_StimulationId_AddedSamplesEnd                    |     0x00008312   |      33554   |
| OVTK_StimulationId_Artifact                           |     0x00008302   |      33538   |
| OVTK_StimulationId_BaselineStart                      |     0x00008007   |      32775   |
| OVTK_StimulationId_BaselineStop                       |     0x00008008   |      32776   |
| OVTK_StimulationId_Beep                               |     0x00008202   |      33282   |
| OVTK_StimulationId_Button1_Pressed                    |     0x00008012   |      32786   |
| OVTK_StimulationId_Button1_Released                   |     0x00008013   |      32787   |
| OVTK_StimulationId_Button2_Pressed                    |     0x00008014   |      32788   |
| OVTK_StimulationId_Button2_Released                   |     0x00008015   |      32789   |
| OVTK_StimulationId_Button3_Pressed                    |     0x00008016   |      32790   |
| OVTK_StimulationId_Button3_Released                   |     0x00008017   |      32791   |
| OVTK_StimulationId_Button4_Pressed                    |     0x00008018   |      32792   |
| OVTK_StimulationId_Button4_Released                   |     0x00008019   |      32793   |
| OVTK_StimulationId_DoubleBeep                         |     0x00008203   |      33283   |
| OVTK_StimulationId_EndOfFile                          |     0x00008204   |      33284   |
| OVTK_StimulationId_ExperimentStart                    |     0x00008001   |      32769   |
| OVTK_StimulationId_ExperimentStop                     |     0x00008002   |      32770   |
| OVTK_StimulationId_Label_00                           |     0x00008100   |      33024   |
| OVTK_StimulationId_Label_01                           |     0x00008101   |      33025   |
| OVTK_StimulationId_Label_02                           |     0x00008102   |      33026   |
| OVTK_StimulationId_Label_03                           |     0x00008103   |      33027   |
| OVTK_StimulationId_Label_04                           |     0x00008104   |      33028   |
| OVTK_StimulationId_Label_05                           |     0x00008105   |      33029   |
| OVTK_StimulationId_Label_06                           |     0x00008106   |      33030   |
| OVTK_StimulationId_Label_07                           |     0x00008107   |      33031   |
| OVTK_StimulationId_Label_08                           |     0x00008108   |      33032   |
| OVTK_StimulationId_Label_09                           |     0x00008109   |      33033   |
| OVTK_StimulationId_Label_0A                           |     0x0000810a   |      33034   |
| OVTK_StimulationId_Label_0B                           |     0x0000810b   |      33035   |
| OVTK_StimulationId_Label_0C                           |     0x0000810c   |      33036   |
| OVTK_StimulationId_Label_0D                           |     0x0000810d   |      33037   |
| OVTK_StimulationId_Label_0E                           |     0x0000810e   |      33038   |
| OVTK_StimulationId_Label_0F                           |     0x0000810f   |      33039   |
| OVTK_StimulationId_Label_10                           |     0x00008110   |      33040   |
| OVTK_StimulationId_Label_11                           |     0x00008111   |      33041   |
| OVTK_StimulationId_Label_12                           |     0x00008112   |      33042   |
| OVTK_StimulationId_Label_13                           |     0x00008113   |      33043   |
| OVTK_StimulationId_Label_14                           |     0x00008114   |      33044   |
| OVTK_StimulationId_Label_15                           |     0x00008115   |      33045   |
| OVTK_StimulationId_Label_16                           |     0x00008116   |      33046   |
| OVTK_StimulationId_Label_17                           |     0x00008117   |      33047   |
| OVTK_StimulationId_Label_18                           |     0x00008118   |      33048   |
| OVTK_StimulationId_Label_19                           |     0x00008119   |      33049   |
| OVTK_StimulationId_Label_1A                           |     0x0000811a   |      33050   |
| OVTK_StimulationId_Label_1B                           |     0x0000811b   |      33051   |
| OVTK_StimulationId_Label_1C                           |     0x0000811c   |      33052   |
| OVTK_StimulationId_Label_1D                           |     0x0000811d   |      33053   |
| OVTK_StimulationId_Label_1E                           |     0x0000811e   |      33054   |
| OVTK_StimulationId_Label_1F                           |     0x0000811f   |      33055   |
| OVTK_StimulationId_Label_20                           |     0x00008120   |      33056   |
| OVTK_StimulationId_Label_21                           |     0x00008121   |      33057   |
| OVTK_StimulationId_Label_22                           |     0x00008122   |      33058   |
| OVTK_StimulationId_Label_23                           |     0x00008123   |      33059   |
| OVTK_StimulationId_Label_24                           |     0x00008124   |      33060   |
| OVTK_StimulationId_Label_25                           |     0x00008125   |      33061   |
| OVTK_StimulationId_Label_26                           |     0x00008126   |      33062   |
| OVTK_StimulationId_Label_27                           |     0x00008127   |      33063   |
| OVTK_StimulationId_Label_28                           |     0x00008128   |      33064   |
| OVTK_StimulationId_Label_29                           |     0x00008129   |      33065   |
| OVTK_StimulationId_Label_2A                           |     0x0000812a   |      33066   |
| OVTK_StimulationId_Label_2B                           |     0x0000812b   |      33067   |
| OVTK_StimulationId_Label_2C                           |     0x0000812c   |      33068   |
| OVTK_StimulationId_Label_2D                           |     0x0000812d   |      33069   |
| OVTK_StimulationId_Label_2E                           |     0x0000812e   |      33070   |
| OVTK_StimulationId_Label_2F                           |     0x0000812f   |      33071   |
| OVTK_StimulationId_Label_30                           |     0x00008130   |      33072   |
| OVTK_StimulationId_Label_31                           |     0x00008131   |      33073   |
| OVTK_StimulationId_Label_32                           |     0x00008132   |      33074   |
| OVTK_StimulationId_Label_33                           |     0x00008133   |      33075   |
| OVTK_StimulationId_Label_34                           |     0x00008134   |      33076   |
| OVTK_StimulationId_Label_35                           |     0x00008135   |      33077   |
| OVTK_StimulationId_Label_36                           |     0x00008136   |      33078   |
| OVTK_StimulationId_Label_37                           |     0x00008137   |      33079   |
| OVTK_StimulationId_Label_38                           |     0x00008138   |      33080   |
| OVTK_StimulationId_Label_39                           |     0x00008139   |      33081   |
| OVTK_StimulationId_Label_3A                           |     0x0000813a   |      33082   |
| OVTK_StimulationId_Label_3B                           |     0x0000813b   |      33083   |
| OVTK_StimulationId_Label_3C                           |     0x0000813c   |      33084   |
| OVTK_StimulationId_Label_3D                           |     0x0000813d   |      33085   |
| OVTK_StimulationId_Label_3E                           |     0x0000813e   |      33086   |
| OVTK_StimulationId_Label_3F                           |     0x0000813f   |      33087   |
| OVTK_StimulationId_Label_40                           |     0x00008140   |      33088   |
| OVTK_StimulationId_Label_41                           |     0x00008141   |      33089   |
| OVTK_StimulationId_Label_42                           |     0x00008142   |      33090   |
| OVTK_StimulationId_Label_43                           |     0x00008143   |      33091   |
| OVTK_StimulationId_Label_44                           |     0x00008144   |      33092   |
| OVTK_StimulationId_Label_45                           |     0x00008145   |      33093   |
| OVTK_StimulationId_Label_46                           |     0x00008146   |      33094   |
| OVTK_StimulationId_Label_47                           |     0x00008147   |      33095   |
| OVTK_StimulationId_Label_48                           |     0x00008148   |      33096   |
| OVTK_StimulationId_Label_49                           |     0x00008149   |      33097   |
| OVTK_StimulationId_Label_4A                           |     0x0000814a   |      33098   |
| OVTK_StimulationId_Label_4B                           |     0x0000814b   |      33099   |
| OVTK_StimulationId_Label_4C                           |     0x0000814c   |      33100   |
| OVTK_StimulationId_Label_4D                           |     0x0000814d   |      33101   |
| OVTK_StimulationId_Label_4E                           |     0x0000814e   |      33102   |
| OVTK_StimulationId_Label_4F                           |     0x0000814f   |      33103   |
| OVTK_StimulationId_Label_50                           |     0x00008150   |      33104   |
| OVTK_StimulationId_Label_51                           |     0x00008151   |      33105   |
| OVTK_StimulationId_Label_52                           |     0x00008152   |      33106   |
| OVTK_StimulationId_Label_53                           |     0x00008153   |      33107   |
| OVTK_StimulationId_Label_54                           |     0x00008154   |      33108   |
| OVTK_StimulationId_Label_55                           |     0x00008155   |      33109   |
| OVTK_StimulationId_Label_56                           |     0x00008156   |      33110   |
| OVTK_StimulationId_Label_57                           |     0x00008157   |      33111   |
| OVTK_StimulationId_Label_58                           |     0x00008158   |      33112   |
| OVTK_StimulationId_Label_59                           |     0x00008159   |      33113   |
| OVTK_StimulationId_Label_5A                           |     0x0000815a   |      33114   |
| OVTK_StimulationId_Label_5B                           |     0x0000815b   |      33115   |
| OVTK_StimulationId_Label_5C                           |     0x0000815c   |      33116   |
| OVTK_StimulationId_Label_5D                           |     0x0000815d   |      33117   |
| OVTK_StimulationId_Label_5E                           |     0x0000815e   |      33118   |
| OVTK_StimulationId_Label_5F                           |     0x0000815f   |      33119   |
| OVTK_StimulationId_Label_60                           |     0x00008160   |      33120   |
| OVTK_StimulationId_Label_61                           |     0x00008161   |      33121   |
| OVTK_StimulationId_Label_62                           |     0x00008162   |      33122   |
| OVTK_StimulationId_Label_63                           |     0x00008163   |      33123   |
| OVTK_StimulationId_Label_64                           |     0x00008164   |      33124   |
| OVTK_StimulationId_Label_65                           |     0x00008165   |      33125   |
| OVTK_StimulationId_Label_66                           |     0x00008166   |      33126   |
| OVTK_StimulationId_Label_67                           |     0x00008167   |      33127   |
| OVTK_StimulationId_Label_68                           |     0x00008168   |      33128   |
| OVTK_StimulationId_Label_69                           |     0x00008169   |      33129   |
| OVTK_StimulationId_Label_6A                           |     0x0000816a   |      33130   |
| OVTK_StimulationId_Label_6B                           |     0x0000816b   |      33131   |
| OVTK_StimulationId_Label_6C                           |     0x0000816c   |      33132   |
| OVTK_StimulationId_Label_6D                           |     0x0000816d   |      33133   |
| OVTK_StimulationId_Label_6E                           |     0x0000816e   |      33134   |
| OVTK_StimulationId_Label_6F                           |     0x0000816f   |      33135   |
| OVTK_StimulationId_Label_70                           |     0x00008170   |      33136   |
| OVTK_StimulationId_Label_71                           |     0x00008171   |      33137   |
| OVTK_StimulationId_Label_72                           |     0x00008172   |      33138   |
| OVTK_StimulationId_Label_73                           |     0x00008173   |      33139   |
| OVTK_StimulationId_Label_74                           |     0x00008174   |      33140   |
| OVTK_StimulationId_Label_75                           |     0x00008175   |      33141   |
| OVTK_StimulationId_Label_76                           |     0x00008176   |      33142   |
| OVTK_StimulationId_Label_77                           |     0x00008177   |      33143   |
| OVTK_StimulationId_Label_78                           |     0x00008178   |      33144   |
| OVTK_StimulationId_Label_79                           |     0x00008179   |      33145   |
| OVTK_StimulationId_Label_7A                           |     0x0000817a   |      33146   |
| OVTK_StimulationId_Label_7B                           |     0x0000817b   |      33147   |
| OVTK_StimulationId_Label_7C                           |     0x0000817c   |      33148   |
| OVTK_StimulationId_Label_7D                           |     0x0000817d   |      33149   |
| OVTK_StimulationId_Label_7E                           |     0x0000817e   |      33150   |
| OVTK_StimulationId_Label_7F                           |     0x0000817f   |      33151   |
| OVTK_StimulationId_Label_80                           |     0x00008180   |      33152   |
| OVTK_StimulationId_Label_81                           |     0x00008181   |      33153   |
| OVTK_StimulationId_Label_82                           |     0x00008182   |      33154   |
| OVTK_StimulationId_Label_83                           |     0x00008183   |      33155   |
| OVTK_StimulationId_Label_84                           |     0x00008184   |      33156   |
| OVTK_StimulationId_Label_85                           |     0x00008185   |      33157   |
| OVTK_StimulationId_Label_86                           |     0x00008186   |      33158   |
| OVTK_StimulationId_Label_87                           |     0x00008187   |      33159   |
| OVTK_StimulationId_Label_88                           |     0x00008188   |      33160   |
| OVTK_StimulationId_Label_89                           |     0x00008189   |      33161   |
| OVTK_StimulationId_Label_8A                           |     0x0000818a   |      33162   |
| OVTK_StimulationId_Label_8B                           |     0x0000818b   |      33163   |
| OVTK_StimulationId_Label_8C                           |     0x0000818c   |      33164   |
| OVTK_StimulationId_Label_8D                           |     0x0000818d   |      33165   |
| OVTK_StimulationId_Label_8E                           |     0x0000818e   |      33166   |
| OVTK_StimulationId_Label_8F                           |     0x0000818f   |      33167   |
| OVTK_StimulationId_Label_90                           |     0x00008190   |      33168   |
| OVTK_StimulationId_Label_91                           |     0x00008191   |      33169   |
| OVTK_StimulationId_Label_92                           |     0x00008192   |      33170   |
| OVTK_StimulationId_Label_93                           |     0x00008193   |      33171   |
| OVTK_StimulationId_Label_94                           |     0x00008194   |      33172   |
| OVTK_StimulationId_Label_95                           |     0x00008195   |      33173   |
| OVTK_StimulationId_Label_96                           |     0x00008196   |      33174   |
| OVTK_StimulationId_Label_97                           |     0x00008197   |      33175   |
| OVTK_StimulationId_Label_98                           |     0x00008198   |      33176   |
| OVTK_StimulationId_Label_99                           |     0x00008199   |      33177   |
| OVTK_StimulationId_Label_9A                           |     0x0000819a   |      33178   |
| OVTK_StimulationId_Label_9B                           |     0x0000819b   |      33179   |
| OVTK_StimulationId_Label_9C                           |     0x0000819c   |      33180   |
| OVTK_StimulationId_Label_9D                           |     0x0000819d   |      33181   |
| OVTK_StimulationId_Label_9E                           |     0x0000819e   |      33182   |
| OVTK_StimulationId_Label_9F                           |     0x0000819f   |      33183   |
| OVTK_StimulationId_Label_A0                           |     0x000081a0   |      33184   |
| OVTK_StimulationId_Label_A1                           |     0x000081a1   |      33185   |
| OVTK_StimulationId_Label_A2                           |     0x000081a2   |      33186   |
| OVTK_StimulationId_Label_A3                           |     0x000081a3   |      33187   |
| OVTK_StimulationId_Label_A4                           |     0x000081a4   |      33188   |
| OVTK_StimulationId_Label_A5                           |     0x000081a5   |      33189   |
| OVTK_StimulationId_Label_A6                           |     0x000081a6   |      33190   |
| OVTK_StimulationId_Label_A7                           |     0x000081a7   |      33191   |
| OVTK_StimulationId_Label_A8                           |     0x000081a8   |      33192   |
| OVTK_StimulationId_Label_A9                           |     0x000081a9   |      33193   |
| OVTK_StimulationId_Label_AA                           |     0x000081aa   |      33194   |
| OVTK_StimulationId_Label_AB                           |     0x000081ab   |      33195   |
| OVTK_StimulationId_Label_AC                           |     0x000081ac   |      33196   |
| OVTK_StimulationId_Label_AD                           |     0x000081ad   |      33197   |
| OVTK_StimulationId_Label_AE                           |     0x000081ae   |      33198   |
| OVTK_StimulationId_Label_AF                           |     0x000081af   |      33199   |
| OVTK_StimulationId_Label_B0                           |     0x000081b0   |      33200   |
| OVTK_StimulationId_Label_B1                           |     0x000081b1   |      33201   |
| OVTK_StimulationId_Label_B2                           |     0x000081b2   |      33202   |
| OVTK_StimulationId_Label_B3                           |     0x000081b3   |      33203   |
| OVTK_StimulationId_Label_B4                           |     0x000081b4   |      33204   |
| OVTK_StimulationId_Label_B5                           |     0x000081b5   |      33205   |
| OVTK_StimulationId_Label_B6                           |     0x000081b6   |      33206   |
| OVTK_StimulationId_Label_B7                           |     0x000081b7   |      33207   |
| OVTK_StimulationId_Label_B8                           |     0x000081b8   |      33208   |
| OVTK_StimulationId_Label_B9                           |     0x000081b9   |      33209   |
| OVTK_StimulationId_Label_BA                           |     0x000081ba   |      33210   |
| OVTK_StimulationId_Label_BB                           |     0x000081bb   |      33211   |
| OVTK_StimulationId_Label_BC                           |     0x000081bc   |      33212   |
| OVTK_StimulationId_Label_BD                           |     0x000081bd   |      33213   |
| OVTK_StimulationId_Label_BE                           |     0x000081be   |      33214   |
| OVTK_StimulationId_Label_BF                           |     0x000081bf   |      33215   |
| OVTK_StimulationId_Label_C0                           |     0x000081c0   |      33216   |
| OVTK_StimulationId_Label_C1                           |     0x000081c1   |      33217   |
| OVTK_StimulationId_Label_C2                           |     0x000081c2   |      33218   |
| OVTK_StimulationId_Label_C3                           |     0x000081c3   |      33219   |
| OVTK_StimulationId_Label_C4                           |     0x000081c4   |      33220   |
| OVTK_StimulationId_Label_C5                           |     0x000081c5   |      33221   |
| OVTK_StimulationId_Label_C6                           |     0x000081c6   |      33222   |
| OVTK_StimulationId_Label_C7                           |     0x000081c7   |      33223   |
| OVTK_StimulationId_Label_C8                           |     0x000081c8   |      33224   |
| OVTK_StimulationId_Label_C9                           |     0x000081c9   |      33225   |
| OVTK_StimulationId_Label_CA                           |     0x000081ca   |      33226   |
| OVTK_StimulationId_Label_CB                           |     0x000081cb   |      33227   |
| OVTK_StimulationId_Label_CC                           |     0x000081cc   |      33228   |
| OVTK_StimulationId_Label_CD                           |     0x000081cd   |      33229   |
| OVTK_StimulationId_Label_CE                           |     0x000081ce   |      33230   |
| OVTK_StimulationId_Label_CF                           |     0x000081cf   |      33231   |
| OVTK_StimulationId_Label_D0                           |     0x000081d0   |      33232   |
| OVTK_StimulationId_Label_D1                           |     0x000081d1   |      33233   |
| OVTK_StimulationId_Label_D2                           |     0x000081d2   |      33234   |
| OVTK_StimulationId_Label_D3                           |     0x000081d3   |      33235   |
| OVTK_StimulationId_Label_D4                           |     0x000081d4   |      33236   |
| OVTK_StimulationId_Label_D5                           |     0x000081d5   |      33237   |
| OVTK_StimulationId_Label_D6                           |     0x000081d6   |      33238   |
| OVTK_StimulationId_Label_D7                           |     0x000081d7   |      33239   |
| OVTK_StimulationId_Label_D8                           |     0x000081d8   |      33240   |
| OVTK_StimulationId_Label_D9                           |     0x000081d9   |      33241   |
| OVTK_StimulationId_Label_DA                           |     0x000081da   |      33242   |
| OVTK_StimulationId_Label_DB                           |     0x000081db   |      33243   |
| OVTK_StimulationId_Label_DC                           |     0x000081dc   |      33244   |
| OVTK_StimulationId_Label_DD                           |     0x000081dd   |      33245   |
| OVTK_StimulationId_Label_DE                           |     0x000081de   |      33246   |
| OVTK_StimulationId_Label_DF                           |     0x000081df   |      33247   |
| OVTK_StimulationId_Label_E0                           |     0x000081e0   |      33248   |
| OVTK_StimulationId_Label_E1                           |     0x000081e1   |      33249   |
| OVTK_StimulationId_Label_E2                           |     0x000081e2   |      33250   |
| OVTK_StimulationId_Label_E3                           |     0x000081e3   |      33251   |
| OVTK_StimulationId_Label_E4                           |     0x000081e4   |      33252   |
| OVTK_StimulationId_Label_E5                           |     0x000081e5   |      33253   |
| OVTK_StimulationId_Label_E6                           |     0x000081e6   |      33254   |
| OVTK_StimulationId_Label_E7                           |     0x000081e7   |      33255   |
| OVTK_StimulationId_Label_E8                           |     0x000081e8   |      33256   |
| OVTK_StimulationId_Label_E9                           |     0x000081e9   |      33257   |
| OVTK_StimulationId_Label_EA                           |     0x000081ea   |      33258   |
| OVTK_StimulationId_Label_EB                           |     0x000081eb   |      33259   |
| OVTK_StimulationId_Label_EC                           |     0x000081ec   |      33260   |
| OVTK_StimulationId_Label_ED                           |     0x000081ed   |      33261   |
| OVTK_StimulationId_Label_EE                           |     0x000081ee   |      33262   |
| OVTK_StimulationId_Label_EF                           |     0x000081ef   |      33263   |
| OVTK_StimulationId_Label_F0                           |     0x000081f0   |      33264   |
| OVTK_StimulationId_Label_F1                           |     0x000081f1   |      33265   |
| OVTK_StimulationId_Label_F2                           |     0x000081f2   |      33266   |
| OVTK_StimulationId_Label_F3                           |     0x000081f3   |      33267   |
| OVTK_StimulationId_Label_F4                           |     0x000081f4   |      33268   |
| OVTK_StimulationId_Label_F5                           |     0x000081f5   |      33269   |
| OVTK_StimulationId_Label_F6                           |     0x000081f6   |      33270   |
| OVTK_StimulationId_Label_F7                           |     0x000081f7   |      33271   |
| OVTK_StimulationId_Label_F8                           |     0x000081f8   |      33272   |
| OVTK_StimulationId_Label_F9                           |     0x000081f9   |      33273   |
| OVTK_StimulationId_Label_FA                           |     0x000081fa   |      33274   |
| OVTK_StimulationId_Label_FB                           |     0x000081fb   |      33275   |
| OVTK_StimulationId_Label_FC                           |     0x000081fc   |      33276   |
| OVTK_StimulationId_Label_FD                           |     0x000081fd   |      33277   |
| OVTK_StimulationId_Label_FE                           |     0x000081fe   |      33278   |
| OVTK_StimulationId_Label_FF                           |     0x000081ff   |      33279   |
| OVTK_StimulationId_NoArtifact                         |     0x00008301   |      33537   |
| OVTK_StimulationId_NonTarget                          |     0x00008206   |      33286   |
| OVTK_StimulationId_Number_00                          |     0x00000000   |          0   |
| OVTK_StimulationId_Number_01                          |     0x00000001   |          1   |
| OVTK_StimulationId_Number_02                          |     0x00000002   |          2   |
| OVTK_StimulationId_Number_03                          |     0x00000003   |          3   |
| OVTK_StimulationId_Number_04                          |     0x00000004   |          4   |
| OVTK_StimulationId_Number_05                          |     0x00000005   |          5   |
| OVTK_StimulationId_Number_06                          |     0x00000006   |          6   |
| OVTK_StimulationId_Number_07                          |     0x00000007   |          7   |
| OVTK_StimulationId_Number_08                          |     0x00000008   |          8   |
| OVTK_StimulationId_Number_09                          |     0x00000009   |          9   |
| OVTK_StimulationId_Number_0A                          |     0x0000000a   |         10   |
| OVTK_StimulationId_Number_0B                          |     0x0000000b   |         11   |
| OVTK_StimulationId_Number_0C                          |     0x0000000c   |         12   |
| OVTK_StimulationId_Number_0D                          |     0x0000000d   |         13   |
| OVTK_StimulationId_Number_0E                          |     0x0000000e   |         14   |
| OVTK_StimulationId_Number_0F                          |     0x0000000f   |         15   |
| OVTK_StimulationId_Number_10                          |     0x00000010   |         16   |
| OVTK_StimulationId_Number_11                          |     0x00000011   |         17   |
| OVTK_StimulationId_Number_12                          |     0x00000012   |         18   |
| OVTK_StimulationId_Number_13                          |     0x00000013   |         19   |
| OVTK_StimulationId_Number_14                          |     0x00000014   |         20   |
| OVTK_StimulationId_Number_15                          |     0x00000015   |         21   |
| OVTK_StimulationId_Number_16                          |     0x00000016   |         22   |
| OVTK_StimulationId_Number_17                          |     0x00000017   |         23   |
| OVTK_StimulationId_Number_18                          |     0x00000018   |         24   |
| OVTK_StimulationId_Number_19                          |     0x00000019   |         25   |
| OVTK_StimulationId_Number_1A                          |     0x0000001a   |         26   |
| OVTK_StimulationId_Number_1B                          |     0x0000001b   |         27   |
| OVTK_StimulationId_Number_1C                          |     0x0000001c   |         28   |
| OVTK_StimulationId_Number_1D                          |     0x0000001d   |         29   |
| OVTK_StimulationId_Number_1E                          |     0x0000001e   |         30   |
| OVTK_StimulationId_Number_1F                          |     0x0000001f   |         31   |
| OVTK_StimulationId_RemovedSamples                     |     0x00008310   |      33552   |
| OVTK_StimulationId_Reset                              |     0x00008208   |      33288   |
| OVTK_StimulationId_RestStart                          |     0x00008009   |      32777   |
| OVTK_StimulationId_RestStop                           |     0x0000800a   |      32778   |
| OVTK_StimulationId_SegmentStart                       |     0x00008003   |      32771   |
| OVTK_StimulationId_SegmentStop                        |     0x00008004   |      32772   |
| OVTK_StimulationId_Target                             |     0x00008205   |      33285   |
| OVTK_StimulationId_ThresholdPassed_Negative           |     0x00008210   |      33296   |
| OVTK_StimulationId_ThresholdPassed_Positive           |     0x00008209   |      33289   |
| OVTK_StimulationId_Train                              |     0x00008201   |      33281   |
| OVTK_StimulationId_TrainCompleted                     |     0x00008207   |      33287   |
| OVTK_StimulationId_TrialStart                         |     0x00008005   |      32773   |
| OVTK_StimulationId_TrialStop                          |     0x00008006   |      32774   |
| OVTK_StimulationId_VisualSteadyStateStimulationStart  |     0x00008010   |      32784   |
| OVTK_StimulationId_VisualSteadyStateStimulationStop   |     0x00008011   |      32785   |
| OVTK_StimulationId_VisualStimulationStart             |     0x0000800b   |      32779   |
| OVTK_StimulationId_VisualStimulationStop              |     0x0000800c   |      32780   |
