# OpenViBE Documentation

![OpenViBE Banner](Banner.png)

## Preface

Welcome to the Doxygen Documentation for OpenViBE @OPENVIBE_MAJOR_VERSION@.@OPENVIBE_MINOR_VERSION@.@OPENVIBE_PATCH_VERSION@.
  
This document has been automatically generated on Linux. Please note

- The documentation lacks components and classes that are not compiled on Linux.
- Some source files do not have doxygen information and may not be represented here.
- Plugins such as boxes and drivers are not included in the class lists to better illustrate the core part of OpenViBE.  
However, you can see the boxes on the [Box Descriptions](_doc__box_algorithms.html) page.

The tutorials and user manual, as well as build instructions for OpenViBE are located on the [website](http://openvibe.inria.fr/documentation-index)

## Online Manual

- [Documentation Index](http://openvibe.inria.fr/documentation-index)
- [User Documentation](http://openvibe.inria.fr/documentation-index/#User+Documentation)
- [Developer Documentation](http://openvibe.inria.fr/documentation-index/#Developer+Documentation)
- [Video Tutorials](http://openvibe.inria.fr/start)
- [Other versions of this doc](http://openvibe.inria.fr/documentation/)

## Quick Access

- [OpenViBE Home](http://openvibe.inria.fr)
- [Forum](http://openvibe.inria.fr/forum)
- [Code Repository](http://gitlab.inria.fr/openvibe)

## Contact

- [Project Contact](http://openvibe.inria.fr/Contact)

This documentation is licenced under CC-BY-SA  
[![cc-by-sa](cc-by-sa.png)](http://creativecommons.org/licenses/by-sa/3.0/legalcode)

\subpage License  
