# OpenViBE Designer {#designer}

\page Designer OpenViBE Designer

The OpenViBE Designer is an authoring tool dedicated to creating and executing Designer scenarios.

− \subpage DesignerIntroduction  
− \subpage DesignerInterface  
− \subpage DesignerAuthoringScenarios  
− \subpage DesignerMetaboxes  
− \subpage DesignerConfiguration
