% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.mtv.R
\name{read.mtv}
\alias{read.mtv}
\title{Import RevMan 4 data files (.mtv)}
\usage{
read.mtv(file)
}
\arguments{
\item{file}{The name of a file to read data values from.}
}
\value{
A data frame containing the following components:
\item{comp.no}{Comparison number.}
\item{outcome.no}{Outcome number.}
\item{group.no}{Group number.}
\item{studlab}{Study label.}
\item{year}{Year of publication.}
\item{event.e}{Number of events in experimental group.}
\item{n.e}{Number of observations in experimental group.}
\item{event.c}{Number of events in control group.}
\item{n.c}{Number of observations in control group.}
\item{mean.e}{Estimated mean in experimental group.}
\item{sd.e}{Standard deviation in experimental group.}
\item{mean.c}{Estimated mean in control group.}
\item{sd.c}{Standard deviation in control group.}
\item{O.E}{Observed minus expected (IPD analysis).}
\item{V}{Variance of \code{O.E} (IPD analysis).}
\item{order}{Ordering of studies.}
\item{conceal}{Concealment of treatment allocation.}
\item{grplab}{Group label.}
\item{type}{Type of outcome. D = dichotomous, C = continuous, P =
  IPD.}
\item{outclab}{Outcome label.}
\item{graph.exp}{Graph label for experimental group.}
\item{graph.cont}{Graph label for control group.}
\item{label.exp}{Label for experimental group.}
\item{label.cont}{Label for control group.}
\item{complab}{Comparison label.}
}
\description{
Reads a file created with RevMan 4 and creates a data frame from
it.
}
\details{
Reads a file created with RevMan 4 (Menu: "File" - "Export" -
"Analysis data file...") and creates a data frame from it.
}
\examples{
# Locate MTV-data file "FLEISS1993.MTV" in sub-directory of R package
# meta
#
filename <- system.file("extdata/FLEISS1993.MTV", package = "meta")
fleiss1933.cc <- read.mtv(filename)

# Same result as R Command example(Fleiss1993bin):
#
metabin(event.e, n.e, event.c, n.c,
  data = fleiss1933.cc, subset = type == "D",
  studlab = paste(studlab, year))

# Same result: example(Fleiss1993cont)
#
metacont(n.e, mean.e, sd.e, n.c, mean.c, sd.c,
  data = fleiss1933.cc, subset = type == "C",
  studlab = paste(studlab, year))

}
\references{
\emph{Review Manager (RevMan)} [Computer program]. Version 4.2.
Copenhagen: The Nordic Cochrane Centre, The Cochrane Collaboration, 2003
}
\seealso{
\code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{datagen}
