% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metabind.R
\name{metabind}
\alias{metabind}
\title{Combine and summarize meta-analysis objects}
\usage{
metabind(
  ...,
  subgroup = NULL,
  name = NULL,
  common = NULL,
  random = NULL,
  prediction = NULL,
  backtransf = NULL,
  outclab = NULL,
  pooled = NULL,
  warn.deprecated = gs("warn.deprecated")
)
}
\arguments{
\item{...}{Any number of meta-analysis objects or a single list
with meta-analyses.}

\item{subgroup}{An optional variable to generate a forest plot with
subgroups.}

\item{name}{An optional character vector providing descriptive
names for the meta-analysis objects.}

\item{common}{A logical vector indicating whether results of common
effect model should be considered.}

\item{random}{A logical vector indicating whether results of random
effects model should be considered.}

\item{prediction}{A logical vector indicating whether results of
prediction intervals should be considered.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots. If
\code{backtransf=TRUE} (default), results for \code{sm="OR"} are
printed as odds ratios rather than log odds ratios, for example.}

\item{outclab}{Outcome label for all meta-analyis objects.}

\item{pooled}{Deprecated argument (replaced by \code{common} and
\code{random}.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}
}
\value{
An object of class \code{c("metabind", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
This function can be used to combine meta-analysis objects and is,
for example, useful to summarize results of various meta-analysis
methods or to generate a forest plot with results of several
subgroup analyses.
}
\details{
This function can be used to combine any number of meta-analysis
objects which is useful, for example, to summarize results of
various meta-analysis methods or to generate a forest plot with
results of several subgroup analyses (see Examples).

Individual study results are not retained with \code{metabind} as
the function allows to combine meta-analyses from different data
sets (e.g., with randomised or observational studies). Individual
study results are retained with R function \code{\link{metamerge}}
which can be used to combine results of meta-analyses of the
same dataset.
}
\examples{
data(Fleiss1993cont)

# Add some (fictitious) grouping variables:
#
Fleiss1993cont$age <- c(55, 65, 55, 65, 55)
Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")

ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD")

# Conduct two subgroup analyses
#
ma1 <- update(ma, subgroup = age, subgroup.name = "Age group")
ma2 <- update(ma, subgroup = region, subgroup.name = "Region")

# Combine random effects subgroup meta-analyses and show forest
# plot with subgroup results
#
mb <- metabind(ma1, ma2, common = FALSE)
mb
forest(mb)

# Use various estimation methods for between-study heterogeneity
# variance
#
ma.pm <- update(ma, method.tau = "PM")
ma.dl <- update(ma, method.tau = "DL")
ma.ml <- update(ma, method.tau = "ML")
ma.hs <- update(ma, method.tau = "HS")
ma.sj <- update(ma, method.tau = "SJ")
ma.he <- update(ma, method.tau = "HE")
ma.eb <- update(ma, method.tau = "EB")

# Combine meta-analyses and show results
#
taus <- c("Restricted maximum-likelihood estimator",
  "Paule-Mandel estimator",
  "DerSimonian-Laird estimator",
  "Maximum-likelihood estimator",
  "Hunter-Schmidt estimator",
  "Sidik-Jonkman estimator",
  "Hedges estimator",
  "Empirical Bayes estimator")
#
ma.taus <- metabind(ma, ma.pm, ma.dl, ma.ml, ma.hs, ma.sj, ma.he, ma.eb,
  name = taus, common = FALSE)
ma.taus
forest(ma.taus)

}
\seealso{
\code{\link{metagen}}, \code{\link{forest.metabind}},
  \code{\link{metamerge}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
