% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Fleiss1993bin.R
\docType{data}
\name{Fleiss1993bin}
\alias{Fleiss1993bin}
\alias{Fleiss93}
\title{Aspirin after Myocardial Infarction}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{year}}\tab year of publication \cr
\bold{\emph{d.asp}}\tab number of deaths in aspirin group \cr
\bold{\emph{n.asp}}\tab number of observations in aspirin group \cr
\bold{\emph{d.plac}}\tab number of deaths in placebo group \cr
\bold{\emph{n.plac}}\tab number of observations in placebo group
}
}
\source{
Fleiss JL (1993):
The statistical basis of meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{2}, 121--45
}
\description{
Meta-analysis on aspirin in preventing death after myocardial
infarction.

Data example in Fleiss (1993) for meta-analysis with binary
outcomes.
}
\examples{
data(Fleiss1993bin)
metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
  studlab = paste(study, year), sm = "OR", random = FALSE)
}
\seealso{
\code{\link{metabin}}
}
\keyword{datasets}
